#!/bin/bash
# osmo-cc-router call processing script

# Disable expansion to process digit '*'
set -f

fn_autodial()
{
	# Automatic dialing
	AUTODIALTIME=0
	[ "$V5IF" = "8198" ] && [ "$V5PORT" = "500" ] && [ "$V5SN" = "20" ] && AUTODIAL="01191" AUTODIALTIME=1

	if [ "$AUTODIAL" != "" ]; then
		#[ "$AUTODIALTIME" -gt 0 ] && echo "alerting" && sleep $AUTODIALTIME
		[ "$AUTODIALTIME" -gt 0 ] && sleep $AUTODIALTIME
		CC_DIALING="$AUTODIAL"
		CC_COMPLETE=1
	fi
}

fn_phonebook()
{
	# Check for complete numbers from phone book, param 1 = number
	# Phonebook file contains one number per line, e.g. 0311
	[ -e /etc/osmocom/phonebook.txt ] && grep -qxf /etc/osmocom/phonebook.txt <<< "$1" && return 0
	return 1
}

fn_blocknumbers()
{
	# Number blocking, param 1 = number
	[[ "$1" =~ ^01[0-4,8-9] ]] 	&& return 0
	[[ "$1" =~ ^090[0-5] ]]		&& return 0
	[[ "$1" =~ ^11[8-9] ]]		&& return 0
	return 1
}

fn_overlapdial()
{
	# Overlap dialing
	echo "overlap"
	DIAL_TO=60
	[ "$CC_DIALING" != "" ] && DIAL_TO=4

	while true; do
		# Detect a completely dialed number
		DIALNUM=$CC_DIALING
		[[ "$DIALNUM" =~ ^0049 ]]			&& DIALNUM="0${DIALNUM:4}"
		[[ "$DIALNUM" =~ ^080000 ]]			&& DIALNUM="*${DIALNUM:6}"
		[[ "$DIALNUM" =~ ^030[1-9] ]]			&& DIALNUM="${DIALNUM:3}"
		[[ "$DIALNUM" =~ ^11[0,2]$ ]]			&& break
		[[ "$DIALNUM" =~ ^01191$ ]]			&& break
		[[ "$DIALNUM" =~ ^01910$ ]]			&& break
		# Phonebook entries
		fn_phonebook "$DIALNUM"                         && break

		# Number not complete, read next digit
		read -t $DIAL_TO msg
		[ $? -gt 128 ] && break

		msg_type=${msg%%\ *}	msg=${msg:${#msg_type}+1}
		msg_digit=${msg%%\ *}	msg=${msg:${#msg_digit}+1}

		[ "$msg_type" != "dialing" ] && echo "disconnect" && exit 0

		# Concat dialed digits
		CC_DIALING="$CC_DIALING$msg_digit"
		DIAL_TO=4
	done
}


fn_metering()
{
	# Determine charging information
	SUBSCR_TYPE="Customer"

	# AN payphone ports
	[[ "$V5PORT" =~ ^30[0,2,3,5,7]$ ]]	&& SUBSCR_TYPE="Payphone"
	[[ "$V5PORT" =~ ^50[2]$ ]]		&& SUBSCR_TYPE="Payphone"

	if [ "$SUBSCR_TYPE" = "Payphone" ] ; then
		# Default values for seconds and initial units
		CHRG_SEC=27
		CHRG_C_UNITS=1

		[[ "$CC_DIALING" =~ ^0 ]]			&& CHRG_SEC=22
		[[ "$CC_DIALING" =~ ^011 ]]			&& CHRG_SEC=15
		[[ "$CC_DIALING" =~ ^01[5-7] ]]		&& CHRG_SEC=20
		[[ "$CC_DIALING" =~ ^031[0-1] ]] 	&& CHRG_SEC=0		CHRG_C_UNITS=0
		[[ "$CC_DIALING" =~ ^0800 ]]		&& CHRG_SEC=0		CHRG_C_UNITS=0
		[[ "$CC_DIALING" =~ ^030 ]]			&& CHRG_SEC=25
		[[ "$CC_DIALING" =~ ^00 ]]			&& CHRG_SEC=10
		[[ "$CC_DIALING" =~ ^0049 ]]		&& CHRG_SEC=22
		[[ "$CC_DIALING" =~ ^11[0,2] ]]		&& CHRG_SEC=0 		CHRG_C_UNITS=0

		[[ "$CC_DIALING" =~ ^\* ]]			&& CHRG_SEC=0		CHRG_C_UNITS=0
		[[ "$CC_DIALING" =~ ^\# ]]			&& CHRG_SEC=0		CHRG_C_UNITS=0
	
	else
		# Standard subscriber ports
		CHRG_SEC=0
		CHRG_C_UNITS=0

		[[ "$CC_DIALING" =~ ^00 ]]			&& CHRG_SEC=30	 	CHRG_C_UNITS=1
		[[ "$CC_DIALING" =~ ^011 ]]			&& CHRG_SEC=15	 	CHRG_C_UNITS=1
	fi

	# Metering test numbers
	[[ "$CC_DIALING" =~ ^030000 ]]			&& CHRG_SEC=0		CHRG_C_UNITS=0		CC_DIALING=01191	# Test free
	[[ "$CC_DIALING" =~ ^030001 ]]			&& CHRG_SEC=2		CC_DIALING=01191						# Test short interval
	[[ "$CC_DIALING" =~ ^030002 ]]			&& CHRG_SEC=480		CC_DIALING=01191						# Test long interval
	[[ "$CC_DIALING" =~ ^030003 ]]			&& CHRG_SEC=0		CC_DIALING=01191						# Test call-setup only 1 unit
	[[ "$CC_DIALING" =~ ^030004 ]]			&& CHRG_SEC=0		CHRG_C_UNITS=2		CC_DIALING=01191	# Test call-setup only 2 units
	[[ "$CC_DIALING" =~ ^030005 ]]			&& CHRG_SEC=2		CHRG_C_UNITS=2		CC_DIALING=01191	# Test call-setup + short interval
	[[ "$CC_DIALING" =~ ^030006 ]]			&& CHRG_SEC=480		CHRG_C_UNITS=2		CC_DIALING=01191	# Test call-setup + long interval

	echo "metering connect_units=$CHRG_C_UNITS period=$CHRG_SEC"
}

# Determine calling interface type
CALLINGNETTYPE="unknown"
[ "$CC_NETWORK_TYPE" = "2" ] && CALLINGNETTYPE="POTS"
[ "$CC_NETWORK_TYPE" = "3" ] && CALLINGNETTYPE="ISDN"
[ "$CC_NETWORK_TYPE" = "4" ] && CALLINGNETTYPE="SIP"

# Calls from SIP to AN
if [ "$CC_CALLING_INTERFACE" = "v5-sip" ] || [ "$CC_CALLING_INTERFACE" = "sip" ] ; then

	V5INFO=$CC_DIALING
	# V5 Access Network Interface
	V5IF=${V5INFO%%-*}	V5INFO=${V5INFO:${#V5IF}+1}
	# V5 Access Port / EFAddr
	V5PORT=${V5INFO%%-*}	V5INFO=${V5INFO:${#V5PORT}+1}
	# Subscriber no (e.g. MSN)
	V5SN=${V5INFO%%-*}	V5INFO=${V5INFO:${#V5SN}+1}

	[ "$V5PORT" = "" ] && V5PORT="$V5IF" V5IF=8198

	#echo "rtp-proxy"
	#echo "called-dtmf"

	[ "$V5SN" = "" ] && echo "call interface=v5-$V5IF-$V5PORT calling=$CC_CALLING"
	[ "$V5SN" != "" ] && echo "call interface=v5-$V5IF-$V5PORT dialing=$V5SN calling=$CC_CALLING"
	
	read -t 15 callprogress
	#[ $? -gt 128 ] && echo "disconnect isdn_cause=41"
	while true; do
		[ "$callprogress" = "call-answer" ] && true
		[ "$callprogress" = "call-proceeding" ] && true
		[ "$callprogress" = "call-disconnect" ] && true
		[[ "$callprogress" =~ ^dtmf ]] && echo "telephone-event ${callprogress:5}"
		read callprogress
	done

# Calls from AN to AN/SIP
else

	V5INFO=$CC_CALLING_INTERFACE
	# v5-8198-500 or 8198-isdn-500
	V5MARK=${V5INFO%%-*}	V5INFO=${V5INFO:${#V5MARK}+1}
	# Bug with ISDN subscribers, interface name wrong
	if [ "$V5MARK" != "v5" ]; then
		# V5 Access Network Interface
		V5IF=$V5MARK
		# (Discard isdn/pstn)
		V5DISCARD=${V5INFO%%-*}	V5INFO=${V5INFO:${#V5DISCARD}+1}
		# V5 Access Port / EFAddr
		V5PORT=${V5INFO%%-*}	V5INFO=${V5INFO:${#V5PORT}+1}
	else
		# V5 Access Network Interface
		V5IF=${V5INFO%%-*}	V5INFO=${V5INFO:${#V5IF}+1}
		# V5 Access Port / EFAddr
		V5PORT=${V5INFO%%-*}	V5INFO=${V5INFO:${#V5PORT}+1}
	fi

	# Subscriber no (e.g. MSN)
	V5SN=$CC_CALLING

	# Automatic dialing
	fn_autodial

	# Collect digits
	if [ "$CC_DIALING" = "" ] || [ "$CC_COMPLETE" = "0" ]; then
		fn_overlapdial
	fi

	# Dialing timeout without digits
	[ "$CC_DIALING" = "" ] && echo "disconnect" && exit 0

	# Pre-Metering Aliases
	[[ "$CC_DIALING" =~ ^0049 ]]				&& CC_DIALING="0${CC_DIALING:4}"
	[[ "$CC_DIALING" =~ ^030[1-9] ]]			&& CC_DIALING="${CC_DIALING:3}"
	[[ "$CC_DIALING" =~ ^080000 ]]				&& CC_DIALING="*${CC_DIALING:6}"
	[[ "$CC_DIALING" =~ ^000 ]]					&& CC_DIALING="*${CC_DIALING:3}"

	# Determine metering information
	fn_metering

	# Post-Metering Aliases
	#[[ "$CC_DIALING" =~ ^190 ]]				&& CC_DIALING="*${CC_DIALING:3}"
	#[[ "$CC_DIALING" =~ ^191 ]]				&& CC_DIALING="#${CC_DIALING:3}"

	# Block certain numbers
	fn_blocknumbers "$CC_DIALING"				&& DIALOUT="DISC"

	# Internal connections
	[[ "$CC_DIALING" =~ ^\#[3][0-9][0-9] ]]		&& DIALOUT="LOCAL" 	UIF="8198"	UPORT="${CC_DIALING:1}"		UMSN=""
	[[ "$CC_DIALING" =~ ^\#[5][0-9][0-9] ]]		&& DIALOUT="LOCAL"	UIF="8198"	UPORT="${CC_DIALING:1:3}"	UMSN="${CC_DIALING:4}"
	#[[ "$CC_DIALING" =~ ^01910 ]]	 			&& DIALOUT="LOCAL"	UIF="8198"	UPORT="308"
	[[ "$CC_DIALING" =~ ^01910 ]]	 			&& DIALOUT=""
	[[ "$CC_DIALING" =~ ^01191 ]]	 			&& DIALOUT=""

	if [ "$DIALOUT" = "LOCAL" ]; then
		# Dialout via Intern

		if [ "$CC_CALLING_PRESENT" = "1" ]; then
			[ "$UMSN" = "" ] && echo "call interface=v5-$UIF-$UPORT no-calling calling-present=$CC_CALLING_PRESENT"
			[ "$UMSN" != "" ] && echo "call interface=v5-$UIF-$UPORT no-calling dialing=$UMSN calling-present=$CC_CALLING_PRESENT"
		else
			[ "$UMSN" = "" ] && echo "call interface=v5-$UIF-$UPORT calling=#$V5PORT$CC_CALLING calling-present=$CC_CALLING_PRESENT"
			[ "$UMSN" != "" ] && echo "call interface=v5-$UIF-$UPORT dialing=$UMSN calling=#$V5PORT$CC_CALLING calling-present=$CC_CALLING_PRESENT"
		fi
		read -t 8 callprogress
		#[ $? -gt 128 ] && echo "disconnect isdn_cause=41" && exit 0
		while true; do
			[ "$callprogress" = "call-answer" ] && true
			[ "$callprogress" = "call-proceeding" ] && true
			[ "$callprogress" = "call-disconnect" ] && true
			read callprogress
		done

	elif [ "$DIALOUT" = "DISC" ]; then
		# Reject call
		echo "disconnect"
		exit 0
	else
		# Dialout via SIP
		# Reject data calls
		if [ "$CC_BEARER_CAPABILITY_NAME" != "speech" ] && [ "$CC_BEARER_CAPABILITY_NAME" != "audio" ] && [ "$CC_BEARER_CAPABILITY_NAME" != "" ]; then
			echo "disconnect"
			exit 0
		fi

		#echo "rtp-proxy"
		#echo "dtmf"

		echo "call interface=v5-sip dialing=$CC_DIALING calling=$V5IF-$V5PORT-$CC_CALLING"

		read -t 8 callprogress
		#[ $? -gt 128 ] && echo "disconnect isdn_cause=41" && exit 0
		while true; do
			[ "$callprogress" = "call-answer" ] && true
			[ "$callprogress" = "call-proceeding" ] && true
			[ "$callprogress" = "call-disconnect" ] && true
			[[ "$callprogress" =~ ^dtmf ]] && echo "called-telephone-event ${callprogress:5}"
			read callprogress
		done
	fi
fi

exit 0
